@@ -54,6 +54,11 @@ try:
 except ImportError:
     xlrd = None
 
+try:
+    import xlwt
+except ImportError:
+    xlwt = None
+
 import wx
 
 from src.config import config
@@ -70,7 +75,7 @@ except ImportError:
     GPG_PRESENT = False
 
 from src.lib.selection import Selection
-from src.lib.fileio import Bz2AOpen
+from src.lib.fileio import AOpen, Bz2AOpen
 
 from src.actions._main_window_actions import Actions
 from src.actions._grid_cell_actions import CellActions
@@ -96,6 +101,12 @@ class FileActions(Actions):
         self.main_window.Bind(self.EVT_CMD_GRID_ACTION_OPEN, self.open)
         self.main_window.Bind(self.EVT_CMD_GRID_ACTION_SAVE, self.save)
 
+        self.type2interface = {
+            "pys": Pys,
+            "pysu": Pys,
+            "xls": Xls,
+        }
+
     def _is_aborted(self, cycle, statustext, total_elements=None, freq=None):
         """Displays progress and returns True if abort
 
@@ -138,7 +149,10 @@ class FileActions(Actions):
 
             # Now wait for the statusbar update to be written on screen
             if is_gtk():
-                wx.Yield()
+                try:
+                    wx.Yield()
+                except:
+                    pass
 
             # Abort if we have to
             if self.need_abort:
@@ -269,6 +283,9 @@ class FileActions(Actions):
         self.code_array.row_heights.clear()
         self.code_array.col_widths.clear()
 
+        # Clear macros
+        self.code_array.macros = ""
+
         # Clear caches
         self.code_array.unredo.reset()
         self.code_array.result_cache.clear()
@@ -284,6 +301,8 @@ class FileActions(Actions):
         ----------
         event.attr: Dict
         \tkey filepath contains file path of file to be loaded
+        \tkey filetype contains file type of file to be loaded
+        \tFiletypes can be pys, pysu, xls
 
         """
 
@@ -292,23 +311,30 @@ class FileActions(Actions):
             filetype = event.attr["filetype"]
 
         except KeyError:
-            filetype = "pys"
+            try:
+                file_ext = filepath.strip().split(".")[-1]
+            except:
+                file_ext = None
+
+            if file_ext in ["pys", "pysu"]:
+                filetype = file_ext
+            else:
+                filetype = "pys"
+
+        type2opener = {
+            "pys": (Bz2AOpen, [filepath, "r"], {"main_window":
+                                                self.main_window}),
+            "pysu": (AOpen, [filepath, "r"], {"main_window": self.main_window})
+        }
 
-        type2opener = {"pys": (Bz2AOpen, [filepath, "r"],
-                               {"main_window": self.main_window})}
         if xlrd is not None:
             type2opener["xls"] = \
                 (xlrd.open_workbook, [filepath], {"formatting_info": True})
 
-        type2interface = {
-            "pys": Pys,
-            "xls": Xls,
-        }
-
         # Specify the interface that shall be used
 
         opener, op_args, op_kwargs = type2opener[filetype]
-        Interface = type2interface[filetype]
+        Interface = self.type2interface[filetype]
 
         # Set state for file open
         self.opening = True
@@ -414,6 +440,14 @@ class FileActions(Actions):
 
         filepath = event.attr["filepath"]
 
+        try:
+            filetype = event.attr["filetype"]
+
+        except KeyError:
+            filetype = "pys"
+
+        Interface = self.type2interface[filetype]
+
         io_error_text = _("Error writing to file {filepath}.")
         io_error_text = io_error_text.format(filepath=filepath)
 
@@ -423,19 +457,35 @@ class FileActions(Actions):
         # Make sure that old save file does not get lost on abort save
         tmpfile = filepath + "~"
 
+        type2opener = {
+            "pys":
+            (Bz2AOpen, [tmpfile, "wb"], {"main_window": self.main_window}),
+            "pysu":
+            (AOpen, [tmpfile, "wb"], {"main_window": self.main_window}),
+        }
+
         try:
             wx.BeginBusyCursor()
             self.grid.Disable()
-            with Bz2AOpen(tmpfile, "wb", main_window=self.main_window) \
-                    as outfile:
 
-                try:
-                    pys = Pys(self.grid.code_array, outfile)
-                    pys.from_code_array()
+            if filetype == "xls":
+                workbook = xlwt.Workbook()
+                interface = Interface(self.grid.code_array, workbook)
+                interface.from_code_array()
+                workbook.save(tmpfile)
 
-                except ValueError, err:
-                    post_command_event(self.main_window, self.StatusBarMsg,
-                                       text=err)
+            else:
+                # Specify the interface that shall be used
+                opener, op_args, op_kwargs = type2opener[filetype]
+
+                with opener(*op_args, **op_kwargs) as outfile:
+                    try:
+                        interface = Interface(self.grid.code_array, outfile)
+                        interface.from_code_array()
+
+                    except ValueError, err:
+                        post_command_event(self.main_window, self.StatusBarMsg,
+                                           text=err)
 
             # Move save file from temp file to filepath
             try:
@@ -457,8 +507,8 @@ class FileActions(Actions):
 
         finally:
             self.saving = False
-            wx.EndBusyCursor()
             self.grid.Enable()
+            wx.EndBusyCursor()
 
         # Mark content as unchanged
         try:
@@ -478,7 +528,7 @@ class FileActions(Actions):
                 # The main window does not exist any more
                 pass
 
-        elif not outfile.aborted:
+        elif filetype in ["pys", "pysu"] and not outfile.aborted:
             self.sign_file(filepath)
 
 
@@ -857,6 +907,68 @@ class TableActions(TableRowActionsMixin, TableColumnActionsMixin,
         self.code_array.unredo.reset()
         self.code_array.result_cache.clear()
 
+    def replace_cells(self, key, sorted_row_idxs):
+        """Replaces cells in current selection so that they are sorted"""
+
+        row, col, tab = key
+
+        new_keys = {}
+        del_keys = []
+
+        selection = self.grid.actions.get_selection()
+
+        for __row, __col, __tab in self.grid.code_array:
+            if __tab == tab and \
+               (not selection or (__row, __col) in selection):
+                new_row = sorted_row_idxs.index(__row)
+                if __row != new_row:
+                    new_keys[(new_row, __col, __tab)] = \
+                        self.grid.code_array((__row, __col, __tab))
+                    del_keys.append((__row, __col, __tab))
+
+        for key in del_keys:
+            self.grid.code_array.pop(key, mark_unredo=False)
+
+        for key in new_keys:
+            self.grid.code_array.__setitem__(key, new_keys[key],
+                                             mark_unredo=False)
+
+        self.grid.code_array.unredo.mark()
+
+    def sort_ascending(self, key):
+        """Sorts selection (or grid if none) corresponding to column of key"""
+
+        row, col, tab = key
+
+        scells = self.grid.code_array[:, col, tab]
+
+        def sorter(i):
+            sorted_ele = scells[i]
+            return sorted_ele is None, sorted_ele
+
+        sorted_row_idxs = sorted(xrange(len(scells)), key=sorter)
+
+        self.replace_cells(key, sorted_row_idxs)
+
+        self.grid.ForceRefresh()
+
+    def sort_descending(self, key):
+        """Sorts inversely selection (or grid if none)
+
+        corresponding to column of key
+
+        """
+
+        row, col, tab = key
+
+        scells = self.grid.code_array[:, col, tab]
+        sorted_row_idxs = sorted(xrange(len(scells)), key=scells.__getitem__)
+        sorted_row_idxs.reverse()
+
+        self.replace_cells(key, sorted_row_idxs)
+
+        self.grid.ForceRefresh()
+
 
 class UnRedoActions(Actions):
     """Undo and redo operations"""
@@ -1110,16 +1222,29 @@ class GridActions(Actions):
 
         """
 
+        shape = self.grid.code_array.shape
+
         if len(value) == 3:
             self.grid._last_selected_cell = row, col, tab = value
+            if row < 0 or col < 0 or tab < 0 or \
+               row >= shape[0] or col >= shape[1] or tab >= shape[2]:
+                raise ValueError("Cell {value} outside of {shape}".format(
+                                 value=value, shape=shape))
 
             if tab != self.cursor[2]:
                 post_command_event(self.main_window,
                                    self.GridActionTableSwitchMsg, newtable=tab)
                 if is_gtk():
-                    wx.Yield()
+                    try:
+                        wx.Yield()
+                    except:
+                        pass
         else:
             row, col = value
+            if row < 0 or col < 0 or row >= shape[0] or col >= shape[1]:
+                raise ValueError("Cell {value} outside of {shape}".format(
+                                 value=value, shape=shape))
+
             self.grid._last_selected_cell = row, col, self.grid.current_table
 
         if not (row is None and col is None):
@@ -1205,6 +1330,25 @@ class SelectionActions(Actions):
 
         self.grid.code_array.result_cache.clear()
 
+    def delete(self):
+        """Deletes a selection if any else deletes the cursor cell
+
+        Refreshes grid after deletion
+
+        """
+
+        if self.grid.IsSelection():
+            # Delete selection
+            self.grid.actions.delete_selection()
+
+        else:
+            # Delete cell at cursor
+            cursor = self.grid.actions.cursor
+            self.grid.actions.delete_cell(cursor)
+
+        # Update grid
+        self.grid.ForceRefresh()
+
     def quote_selection(self):
         """Quotes selected cells, marks content as changed"""
 
@@ -1219,6 +1363,33 @@ class SelectionActions(Actions):
 
         self.grid.code_array.result_cache.clear()
 
+    def copy_selection_access_string(self):
+        """Copys access_string to selection to the clipboard
+
+        An access string is Python code to reference the selection
+        If there is no selection then a reference to the current cell is copied
+
+        """
+
+        selection = self.get_selection()
+        if not selection:
+            cursor = self.grid.actions.cursor
+            selection = Selection([], [], [], [], [tuple(cursor[:2])])
+        shape = self.grid.code_array.shape
+        tab = self.grid.current_table
+
+        access_string = selection.get_access_string(shape, tab)
+
+        # Copy access string to clipboard
+        self.grid.main_window.clipboard.set_clipboard(access_string)
+
+        # Display copy operation and access string in status bar
+        statustext = _("Cell reference copied to clipboard: {access_string}")
+        statustext = statustext.format(access_string=access_string)
+
+        post_command_event(self.main_window, self.StatusBarMsg,
+                           text=statustext)
+
 
 class FindActions(Actions):
     """Actions for finding inside the grid"""
@@ -1323,6 +1494,8 @@ class FindActions(Actions):
         post_command_event(self.main_window, self.StatusBarMsg,
                            text=statustext)
 
+        self.grid.ForceRefresh()
+
     def replace(self, findpos, find_string, replace_string):
         """Replaces occurrences of find_string with replace_string at findpos
 
