@@ -31,7 +31,7 @@ import wx
 
 ##from sysvars import get_color, get_font_string
 
-VERSION = "0.2.7"
+VERSION = "0.3.0"
 
 
 class DefaultConfig(object):
@@ -75,6 +75,8 @@ class DefaultConfig(object):
 
         self.max_unredo = "5000"
 
+        self.timer_interval = "1000"
+
         # Maximum result length in a cell in characters
         self.max_result_length = "1000"
 
@@ -104,7 +106,8 @@ class DefaultConfig(object):
         # GPG parameters
         # --------------
 
-        self.gpg_key_uid = repr('')
+        #self.gpg_key_uid = repr('')  # Deprecated
+        self.gpg_key_fingerprint = repr('')
 
         # CSV parameters for import and export
         # ------------------------------------
@@ -181,12 +184,13 @@ class Config(object):
                 setattr(self.data, key, getattr(DefaultConfig(), key))
             self.data.config_version = self.version
 
-        # Reset GPG key if version prior 0.2.4
-        # because of pyme -> gnupg migration
+        # Delete gpg_key_uid and insert fingerprint key
 
-        if old_config:
-            key = "gpg_key_uid"
-            setattr(self.data, key, getattr(DefaultConfig(), key))
+        if hasattr(self.data, "gpg_key_uid"):
+            oldkey = "gpg_key_uid"
+            delattr(self.data, oldkey)
+            newkey = "gpg_key_fingerprint"
+            setattr(self.data, newkey, getattr(DefaultConfig(), newkey))
 
     def save(self):
         """Saves configuration file"""
